import React from "react";
import { DragDropContext, Droppable, Draggable } from "react-beautiful-dnd";
// credit: https://codesandbox.io/s/ql08j35j3q?file=/index.js:0-6365

import { ListGroup } from "react-bootstrap";
//import Loading from "../Loading";
//import moment from "moment";

interface IProps {}
interface IState {}

class Operations extends React.Component<IProps, IState> {
  constructor(props: IProps) {
    super(props);
    this.state = {
      tasks: [`develop`, `deploy`, `review`],
    };

    this.handleChange = this.handleChange.bind(this);
  }

  componentDidMount() {
    const listItens = document.querySelectorAll(".draggable");
    [].forEach.call(listItens, function (item) {
      addEventsDragAndDrop(item);
    });
    //btn.addEventListener("click", addNewItem);
  }

  // fake data generator
  getItems = (count, offset = 0) =>
    Array.from({ length: count }, (v, k) => k).map((k) => ({
      id: `item-${k + offset}`,
      content: `item ${k + offset}`,
    }));

  // a little function to help us with reordering the result
  reorder = (list, startIndex, endIndex) => {
    const result = Array.from(list);
    const [removed] = result.splice(startIndex, 1);
    result.splice(endIndex, 0, removed);

    return result;
  };

  /**
   * Moves an item from one list to another list.
   */
  move = (source, destination, droppableSource, droppableDestination) => {
    const sourceClone = Array.from(source);
    const destClone = Array.from(destination);
    const [removed] = sourceClone.splice(droppableSource.index, 1);

    destClone.splice(droppableDestination.index, 0, removed);

    const result = {};
    result[droppableSource.droppableId] = sourceClone;
    result[droppableDestination.droppableId] = destClone;

    return result;
  };

  grid = 8;

  getItemStyle = (isDragging, draggableStyle) => ({
    // some basic styles to make the items look a bit nicer
    userSelect: "none",
    padding: grid * 2,
    margin: `0 0 ${grid}px 0`,

    // change background colour if dragging
    background: isDragging ? "lightgreen" : "grey",

    // styles we need to apply on draggables
    ...draggableStyle,
  });

  getListStyle = (isDraggingOver) => ({
    background: isDraggingOver ? "lightblue" : "lightgrey",
    padding: grid,
    width: 250,
  });

  /**
   * A semi-generic way to handle multiple lists. Matches
   * the IDs of the droppable container to the names of the
   * source arrays stored in the state.
   */
  id2List = {
    droppable: "items",
    droppable2: "selected",
  };

  getList = (id) => this.state[this.id2List[id]];

  onDragEnd = (result) => {
    const { source, destination } = result;

    // dropped outside the list
    if (!destination) {
      return;
    }

    if (source.droppableId === destination.droppableId) {
      const items = reorder(
        this.getList(source.droppableId),
        source.index,
        destination.index
      );

      let state = { items };

      if (source.droppableId === "droppable2") {
        state = { selected: items };
      }

      this.setState(state);
    } else {
      const result = move(
        this.getList(source.droppableId),
        this.getList(destination.droppableId),
        source,
        destination
      );

      this.setState({
        items: result.droppable,
        selected: result.droppable2,
      });
    }
  };

  handleChange(e) {
    this.setState({ [e.target.name]: e.target.value });
  }

  render() {
    const { tasks } = this.state;
    //const {  } = this.props;

    //if (!tasks.length) return <Loading target="tasks" />;

    return (
      <div className="p-3 text-light">
        <h2>Operations Tasks</h2>
        <ul>
          <li class="draggable" draggable="true">
            HTML
          </li>
          <li class="draggable" draggable="true">
            CSS
          </li>
          <li class="draggable" draggable="true">
            JavaScript
          </li>
          <li class="draggable" draggable="true">
            PHP
          </li>
          <li class="draggable" draggable="true">
            MySQL
          </li>
        </ul>
        <div className="w-100 d-flex flex-row justify-content-fill">
          <div className="col-3 d-flex flex-column">
            <h3>Backlog</h3>
            <ListGroup>
              {tasks.map((t) => (
                <ListGroup.Item className={`text-dark`}>{t}</ListGroup.Item>
              ))}
            </ListGroup>
          </div>
          <div className="col-3 d-flex flex-column">
            <h3>In progress</h3>
            <ListGroup>
              {tasks.map((t) => (
                <ListGroup.Item className={`text-dark`}>{t}</ListGroup.Item>
              ))}
            </ListGroup>
          </div>
          <div className="col-3 d-flex flex-column">
            <h3>In review</h3>
            <ListGroup>
              {tasks.map((t) => (
                <ListGroup.Item className={`text-dark`}>{t}</ListGroup.Item>
              ))}
            </ListGroup>
          </div>
          <div className="col-3 d-flex flex-column">
            <h3>Done</h3>
            <ListGroup>
              {tasks.map((t) => (
                <ListGroup.Item className={`text-dark`}>{t}</ListGroup.Item>
              ))}
            </ListGroup>
          </div>
        </div>
        <DragDropContext onDragEnd={this.onDragEnd}>
          <Droppable droppableId="droppable">
            {(provided, snapshot) => (
              <div
                ref={provided.innerRef}
                style={this.getListStyle(snapshot.isDraggingOver)}
              >
                {this.state.items.map((item, index) => (
                  <Draggable key={item.id} draggableId={item.id} index={index}>
                    {(provided, snapshot) => (
                      <div
                        ref={provided.innerRef}
                        {...provided.draggableProps}
                        {...provided.dragHandleProps}
                        style={getItemStyle(
                          snapshot.isDragging,
                          provided.draggableProps.style
                        )}
                      >
                        {item.content}
                      </div>
                    )}
                  </Draggable>
                ))}
                {provided.placeholder}
              </div>
            )}
          </Droppable>
          <Droppable droppableId="droppable2">
            {(provided, snapshot) => (
              <div
                ref={provided.innerRef}
                style={this.getListStyle(snapshot.isDraggingOver)}
              >
                {this.state.selected.map((item, index) => (
                  <Draggable key={item.id} draggableId={item.id} index={index}>
                    {(provided, snapshot) => (
                      <div
                        ref={provided.innerRef}
                        {...provided.draggableProps}
                        {...provided.dragHandleProps}
                        style={getItemStyle(
                          snapshot.isDragging,
                          provided.draggableProps.style
                        )}
                      >
                        {item.content}
                      </div>
                    )}
                  </Draggable>
                ))}
                {provided.placeholder}
              </div>
            )}
          </Droppable>
        </DragDropContext>
        );
        <div className="position-fixed" style={{ right: "0px", top: "0px" }}>
          <a href="/">Back</a>
        </div>
      </div>
    );
  }
}

export default Operations;
